@@ -11,8 +11,8 @@ import (
 	synccommon "github.com/argoproj/gitops-engine/pkg/sync/common"
 	"github.com/argoproj/gitops-engine/pkg/utils/kube/kubetest"
 	"github.com/argoproj/pkg/sync"
-	"github.com/dgrijalva/jwt-go/v4"
 	"github.com/ghodss/yaml"
+	"github.com/golang-jwt/jwt/v4"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/mock"
 	"github.com/stretchr/testify/require"
@@ -41,6 +41,7 @@ import (
 	"github.com/argoproj/argo-cd/v2/util/cache"
 	"github.com/argoproj/argo-cd/v2/util/db"
 	"github.com/argoproj/argo-cd/v2/util/errors"
+	"github.com/argoproj/argo-cd/v2/util/grpc"
 	"github.com/argoproj/argo-cd/v2/util/rbac"
 	"github.com/argoproj/argo-cd/v2/util/settings"
 )
@@ -72,6 +73,36 @@ func fakeAppList() *apiclient.AppList {
 	}
 }
 
+func fakeResolveRevesionResponse() *apiclient.ResolveRevisionResponse {
+	return &apiclient.ResolveRevisionResponse{
+		Revision:          "f9ba9e98119bf8c1176fbd65dbae26a71d044add",
+		AmbiguousRevision: "HEAD (f9ba9e98119bf8c1176fbd65dbae26a71d044add)",
+	}
+}
+
+func fakeResolveRevesionResponseHelm() *apiclient.ResolveRevisionResponse {
+	return &apiclient.ResolveRevisionResponse{
+		Revision:          "0.7.*",
+		AmbiguousRevision: "0.7.* (0.7.2)",
+	}
+}
+
+func fakeRepoServerClient(isHelm bool) *mocks.RepoServerServiceClient {
+	mockRepoServiceClient := mocks.RepoServerServiceClient{}
+	mockRepoServiceClient.On("ListApps", mock.Anything, mock.Anything).Return(fakeAppList(), nil)
+	mockRepoServiceClient.On("GenerateManifest", mock.Anything, mock.Anything).Return(&apiclient.ManifestResponse{}, nil)
+	mockRepoServiceClient.On("GetAppDetails", mock.Anything, mock.Anything).Return(&apiclient.RepoAppDetailsResponse{}, nil)
+	mockRepoServiceClient.On("TestRepository", mock.Anything, mock.Anything).Return(&apiclient.TestRepositoryResponse{}, nil)
+
+	if isHelm {
+		mockRepoServiceClient.On("ResolveRevision", mock.Anything, mock.Anything).Return(fakeResolveRevesionResponseHelm(), nil)
+	} else {
+		mockRepoServiceClient.On("ResolveRevision", mock.Anything, mock.Anything).Return(fakeResolveRevesionResponse(), nil)
+	}
+
+	return &mockRepoServiceClient
+}
+
 // return an ApplicationServiceServer which returns fake data
 func newTestAppServer(objects ...runtime.Object) *Server {
 	f := func(enf *rbac.Enforcer) {
@@ -107,13 +138,7 @@ func newTestAppServerWithEnforcerConfigure(f func(*rbac.Enforcer), objects ...ru
 	_, err = db.CreateCluster(ctx, fakeCluster())
 	errors.CheckError(err)
 
-	mockRepoServiceClient := mocks.RepoServerServiceClient{}
-	mockRepoServiceClient.On("ListApps", mock.Anything, mock.Anything).Return(fakeAppList(), nil)
-	mockRepoServiceClient.On("GenerateManifest", mock.Anything, mock.Anything).Return(&apiclient.ManifestResponse{}, nil)
-	mockRepoServiceClient.On("GetAppDetails", mock.Anything, mock.Anything).Return(&apiclient.RepoAppDetailsResponse{}, nil)
-	mockRepoServiceClient.On("TestRepository", mock.Anything, mock.Anything).Return(&apiclient.TestRepositoryResponse{}, nil)
-
-	mockRepoClient := &mocks.Clientset{RepoServerServiceClient: &mockRepoServiceClient}
+	mockRepoClient := &mocks.Clientset{RepoServerServiceClient: fakeRepoServerClient(false)}
 
 	defaultProj := &appsv1.AppProject{
 		ObjectMeta: metav1.ObjectMeta{Name: "default", Namespace: "default"},
@@ -172,7 +197,7 @@ func newTestAppServerWithEnforcerConfigure(f func(*rbac.Enforcer), objects ...ru
 		panic("Timed out waiting for caches to sync")
 	}
 
-	server := NewServer(
+	server, _ := NewServer(
 		testNamespace,
 		kubeclientset,
 		fakeAppsClientset,
@@ -336,7 +361,7 @@ func TestCreateApp(t *testing.T) {
 	appServer := newTestAppServer()
 	testApp.Spec.Project = ""
 	createReq := application.ApplicationCreateRequest{
-		Application: *testApp,
+		Application: testApp,
 	}
 	app, err := appServer.Create(context.Background(), &createReq)
 	assert.NoError(t, err)
@@ -349,7 +374,7 @@ func TestCreateAppWithDestName(t *testing.T) {
 	appServer := newTestAppServer()
 	testApp := newTestAppWithDestName()
 	createReq := application.ApplicationCreateRequest{
-		Application: *testApp,
+		Application: testApp,
 	}
 	app, err := appServer.Create(context.Background(), &createReq)
 	assert.NoError(t, err)
@@ -374,7 +399,7 @@ func TestUpdateAppSpec(t *testing.T) {
 	testApp.Spec.Project = ""
 	spec, err := appServer.UpdateSpec(context.Background(), &application.ApplicationUpdateSpecRequest{
 		Name: &testApp.Name,
-		Spec: testApp.Spec,
+		Spec: &testApp.Spec,
 	})
 	assert.NoError(t, err)
 	assert.Equal(t, "default", spec.Project)
@@ -387,7 +412,7 @@ func TestDeleteApp(t *testing.T) {
 	ctx := context.Background()
 	appServer := newTestAppServer()
 	createReq := application.ApplicationCreateRequest{
-		Application: *newTestApp(),
+		Application: newTestApp(),
 	}
 	app, err := appServer.Create(ctx, &createReq)
 	assert.Nil(t, err)
@@ -487,11 +512,10 @@ func TestSyncAndTerminate(t *testing.T) {
 	testApp := newTestApp()
 	testApp.Spec.Source.RepoURL = "https://github.com/argoproj/argo-cd.git"
 	createReq := application.ApplicationCreateRequest{
-		Application: *testApp,
+		Application: testApp,
 	}
 	app, err := appServer.Create(ctx, &createReq)
 	assert.Nil(t, err)
-
 	app, err = appServer.Sync(ctx, &application.ApplicationSyncRequest{Name: &app.Name})
 	assert.Nil(t, err)
 	assert.NotNil(t, app)
@@ -531,7 +555,9 @@ func TestSyncHelm(t *testing.T) {
 	testApp.Spec.Source.Chart = "argo-cd"
 	testApp.Spec.Source.TargetRevision = "0.7.*"
 
-	app, err := appServer.Create(ctx, &application.ApplicationCreateRequest{Application: *testApp})
+	appServer.repoClientset = &mocks.Clientset{RepoServerServiceClient: fakeRepoServerClient(true)}
+
+	app, err := appServer.Create(ctx, &application.ApplicationCreateRequest{Application: testApp})
 	assert.NoError(t, err)
 
 	app, err = appServer.Sync(ctx, &application.ApplicationSyncRequest{Name: &app.Name})
@@ -551,7 +577,7 @@ func TestSyncGit(t *testing.T) {
 	testApp.Spec.Source.RepoURL = "https://github.com/org/test"
 	testApp.Spec.Source.Path = "deploy"
 	testApp.Spec.Source.TargetRevision = "0.7.*"
-	app, err := appServer.Create(ctx, &application.ApplicationCreateRequest{Application: *testApp})
+	app, err := appServer.Create(ctx, &application.ApplicationCreateRequest{Application: testApp})
 	assert.NoError(t, err)
 	syncReq := &application.ApplicationSyncRequest{
 		Name: &app.Name,
@@ -583,7 +609,7 @@ func TestRollbackApp(t *testing.T) {
 
 	updatedApp, err := appServer.Rollback(context.Background(), &application.ApplicationRollbackRequest{
 		Name: &testApp.Name,
-		ID:   1,
+		Id:   pointer.Int64(1),
 	})
 
 	assert.Nil(t, err)
@@ -618,7 +644,9 @@ p, admin, applications, update, default/test-app, allow
 p, admin, applications, update, my-proj/test-app, allow
 `)
 	_, err = appServer.Update(ctx, &application.ApplicationUpdateRequest{Application: testApp})
-	assert.Equal(t, status.Code(err), codes.PermissionDenied)
+	statusErr := grpc.UnwrapGRPCStatus(err)
+	assert.NotNil(t, statusErr)
+	assert.Equal(t, codes.PermissionDenied, statusErr.Code())
 
 	// Verify inability to change projects without update privileges in new project
 	_ = appServer.enf.SetBuiltinPolicy(`
@@ -634,7 +662,9 @@ p, admin, applications, create, my-proj/test-app, allow
 p, admin, applications, update, my-proj/test-app, allow
 `)
 	_, err = appServer.Update(ctx, &application.ApplicationUpdateRequest{Application: testApp})
-	assert.Equal(t, status.Code(err), codes.PermissionDenied)
+	statusErr = grpc.UnwrapGRPCStatus(err)
+	assert.NotNil(t, statusErr)
+	assert.Equal(t, codes.PermissionDenied, statusErr.Code())
 
 	// Verify can update project with proper permissions
 	_ = appServer.enf.SetBuiltinPolicy(`
@@ -655,19 +685,19 @@ func TestAppJsonPatch(t *testing.T) {
 	appServer := newTestAppServer(testApp)
 	appServer.enf.SetDefaultRole("")
 
-	app, err := appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: "garbage"})
+	app, err := appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: pointer.String("garbage")})
 	assert.Error(t, err)
 	assert.Nil(t, app)
 
-	app, err = appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: "[]"})
+	app, err = appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: pointer.String("[]")})
 	assert.NoError(t, err)
 	assert.NotNil(t, app)
 
-	app, err = appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: `[{"op": "replace", "path": "/spec/source/path", "value": "foo"}]`})
+	app, err = appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: pointer.String(`[{"op": "replace", "path": "/spec/source/path", "value": "foo"}]`)})
 	assert.NoError(t, err)
 	assert.Equal(t, "foo", app.Spec.Source.Path)
 
-	app, err = appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: `[{"op": "remove", "path": "/metadata/annotations/test.annotation"}]`})
+	app, err = appServer.Patch(ctx, &application.ApplicationPatchRequest{Name: &testApp.Name, Patch: pointer.String(`[{"op": "remove", "path": "/metadata/annotations/test.annotation"}]`)})
 	assert.NoError(t, err)
 	assert.NotContains(t, app.Annotations, "test.annotation")
 }
@@ -681,7 +711,7 @@ func TestAppMergePatch(t *testing.T) {
 	appServer.enf.SetDefaultRole("")
 
 	app, err := appServer.Patch(ctx, &application.ApplicationPatchRequest{
-		Name: &testApp.Name, Patch: `{"spec": { "source": { "path": "foo" } }}`, PatchType: "merge"})
+		Name: &testApp.Name, Patch: pointer.String(`{"spec": { "source": { "path": "foo" } }}`), PatchType: pointer.String("merge")})
 	assert.NoError(t, err)
 	assert.Equal(t, "foo", app.Spec.Source.Path)
 }
@@ -721,16 +751,13 @@ func TestGetCachedAppState(t *testing.T) {
 	testApp.ObjectMeta.ResourceVersion = "1"
 	testApp.Spec.Project = "none"
 	appServer := newTestAppServer(testApp)
-
 	fakeClientSet := appServer.appclientset.(*apps.Clientset)
-
 	t.Run("NoError", func(t *testing.T) {
 		err := appServer.getCachedAppState(context.Background(), testApp, func() error {
 			return nil
 		})
 		assert.NoError(t, err)
 	})
-
 	t.Run("CacheMissErrorTriggersRefresh", func(t *testing.T) {
 		retryCount := 0
 		patched := false
@@ -872,7 +899,7 @@ func TestLogsGetSelectedPod(t *testing.T) {
 }
 
 // refreshAnnotationRemover runs an infinite loop until it detects and removes refresh annotation or given context is done
-func refreshAnnotationRemover(t *testing.T, ctx context.Context, patched *int32, appServer *Server, appName string) {
+func refreshAnnotationRemover(t *testing.T, ctx context.Context, patched *int32, appServer *Server, appName string, ch chan string) {
 	for ctx.Err() == nil {
 		a, err := appServer.appLister.Get(appName)
 		require.NoError(t, err)
@@ -884,7 +911,7 @@ func refreshAnnotationRemover(t *testing.T, ctx context.Context, patched *int32,
 				context.Background(), a, metav1.UpdateOptions{})
 			require.NoError(t, err)
 			atomic.AddInt32(patched, 1)
-			break
+			ch <- ""
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
@@ -899,20 +926,28 @@ func TestGetAppRefresh_NormalRefresh(t *testing.T) {
 
 	var patched int32
 
-	go refreshAnnotationRemover(t, ctx, &patched, appServer, testApp.Name)
+	ch := make(chan string, 1)
+
+	go refreshAnnotationRemover(t, ctx, &patched, appServer, testApp.Name, ch)
 
 	_, err := appServer.Get(context.Background(), &application.ApplicationQuery{
 		Name:    &testApp.Name,
 		Refresh: pointer.StringPtr(string(appsv1.RefreshTypeNormal)),
 	})
 	assert.NoError(t, err)
-	assert.Equal(t, atomic.LoadInt32(&patched), int32(1))
+
+	select {
+	case <-ch:
+		assert.Equal(t, atomic.LoadInt32(&patched), int32(1))
+	case <-time.After(10 * time.Second):
+		assert.Fail(t, "Out of time ( 10 seconds )")
+	}
+
 }
 
 func TestGetAppRefresh_HardRefresh(t *testing.T) {
 	ctx, cancel := context.WithCancel(context.Background())
 	defer cancel()
-
 	testApp := newTestApp()
 	testApp.ObjectMeta.ResourceVersion = "1"
 	appServer := newTestAppServer(testApp)
@@ -927,15 +962,24 @@ func TestGetAppRefresh_HardRefresh(t *testing.T) {
 
 	var patched int32
 
-	go refreshAnnotationRemover(t, ctx, &patched, appServer, testApp.Name)
+	ch := make(chan string, 1)
+
+	go refreshAnnotationRemover(t, ctx, &patched, appServer, testApp.Name, ch)
 
 	_, err := appServer.Get(context.Background(), &application.ApplicationQuery{
 		Name:    &testApp.Name,
 		Refresh: pointer.StringPtr(string(appsv1.RefreshTypeHard)),
 	})
 	assert.NoError(t, err)
-	assert.Equal(t, atomic.LoadInt32(&patched), int32(1))
 	require.NotNil(t, getAppDetailsQuery)
 	assert.True(t, getAppDetailsQuery.NoCache)
 	assert.Equal(t, &testApp.Spec.Source, getAppDetailsQuery.Source)
+
+	assert.NoError(t, err)
+	select {
+	case <-ch:
+		assert.Equal(t, atomic.LoadInt32(&patched), int32(1))
+	case <-time.After(10 * time.Second):
+		assert.Fail(t, "Out of time ( 10 seconds )")
+	}
 }
